import torch
import torch.nn as nn


class FixedGru(nn.Module):
    # Custom GRU that matches the implementation provided by Reed et al.

    def __init__(self, num_steps, emb_dim):
        super().__init__()
        # update gate
        self.i2h_update = nn.Linear(emb_dim, emb_dim)
        self.h2h_update = nn.Linear(emb_dim, emb_dim)

        # reset gate
        self.i2h_reset = nn.Linear(emb_dim, emb_dim)
        self.h2h_reset = nn.Linear(emb_dim, emb_dim)

        # candidate hidden state
        self.i2h = nn.Linear(emb_dim, emb_dim)
        self.h2h = nn.Linear(emb_dim, emb_dim)

        self.num_steps = num_steps

    def forward(self, txt):
        res = []
        res_intermediate = []
        for i in range(self.num_steps):
            if i == 0:
                output = torch.tanh(self.i2h(txt[:, i])).unsqueeze(1)
            else:
                # compute update and reset gates
                update = torch.sigmoid(self.i2h_update(txt[:, i]) + \
                                       self.h2h_update(res[i - 1]))
                reset = torch.sigmoid(self.i2h_reset(txt[:, i]) + \
                                      self.h2h_reset(res[i - 1]))

                # compute candidate hidden state
                gated_hidden = reset * res[i - 1]
                p1 = self.i2h(txt[:, i])
                p2 = self.h2h(gated_hidden)
                hidden_cand = torch.tanh(p1 + p2)

                # use gates to interpolate hidden state
                zh = update * hidden_cand
                zhm1 = ((update * -1) + 1) * res[i - 1]
                output = zh + zhm1

            res.append(output)
            res_intermediate.append(output)

        res = torch.cat(res, dim=1)
        res = torch.mean(res, dim=1)
        return res



import torch as th
from torch.utils.data import Dataset
import pandas as pd
import os
import numpy as np
import ffmpeg

class VideoLoader(Dataset):
    """Pytorch video loader."""

    def __init__(
            self,
            csv,
            framerate=1,
            size=112,
            centercrop=False,
    ):
        """
        Args:
        """
        self.csv = pd.read_csv(csv)
        self.centercrop = centercrop
        self.size = size
        self.framerate = framerate

    def __len__(self):
        return len(self.csv)

    def _get_video_dim(self, video_path):
        probe = ffmpeg.probe(video_path)
        video_stream = next((stream for stream in probe['streams']
                             if stream['codec_type'] == 'video'), None)
        width = int(video_stream['width'])
        height = int(video_stream['height'])
        return height, width

    def _get_output_dim(self, h, w):
        if isinstance(self.size, tuple) and len(self.size) == 2:
            return self.size
        elif h >= w:
            return int(h * self.size / w), self.size
        else:
            return self.size, int(w * self.size / h)

    def __getitem__(self, idx):
        video_path = self.csv['video_path'].values[idx]
        output_file = self.csv['feature_path'].values[idx]

        if not(os.path.isfile(output_file)) and os.path.isfile(video_path):
            print('Decoding video: {}'.format(video_path))
            try:
                h, w = self._get_video_dim(video_path)
            except:
                print('ffprobe failed at: {}'.format(video_path))
                return {'video': th.zeros(1), 'input': video_path,
                        'output': output_file}
            height, width = self._get_output_dim(h, w)
            cmd = (
                ffmpeg
                .input(video_path)
                .filter('fps', fps=self.framerate)
                .filter('scale', width, height)
            )
            if self.centercrop:
                x = int((width - self.size) / 2.0)
                y = int((height - self.size) / 2.0)
                cmd = cmd.crop(x, y, self.size, self.size)
            out, _ = (
                cmd.output('pipe:', format='rawvideo', pix_fmt='rgb24')
                .run(capture_stdout=True, quiet=True)
            )
            if self.centercrop and isinstance(self.size, int):
                height, width = self.size, self.size
            video = np.frombuffer(out, np.uint8).reshape([-1, height, width, 3])
            video = th.from_numpy(video.astype('float32'))
            video = video.permute(0, 3, 1, 2)
        else:
            video = th.zeros(1)
            
        return {'video': video, 'input': video_path, 'output': output_file}



from mmkfeatures.fusion import log
from mmkfeatures.fusion.metadataconfigs import *
from tqdm import tqdm

#this function checks the heirarchy format of a given computatioanl sequence data. This will crash the program if data is in wrong format. If in correct format the return value is simply True 
def validate_data_format(data,root_name,verbose=True):
	log.status("Checking the format of the data in <%s> computational sequence ..."%root_name)

	pbar = log.progress_bar(total=len(data.keys()),unit=" Computational Sequence Entries",leave=False)
	failure=False
	if (type(data) is not dict):
		#this will cause the rest of the pipeline to crash - RuntimeError
		log.error("%s computational sequence data is not in heirarchy format ...",error=True)
	try:

		#for each video check the shapes of the intervals and features
		for vid in data.keys():
			if "F" in data[vid].keys():
				# check the intervals first - if failure simply show a warning - no exit since we want to identify all the cases
				if (len(data[vid]["ITV"]) != 0 and len(data[vid]["ITV"].shape) != 2):
					if verbose: log.error(
						"Timestamped content (e.g. video,audio) <%s> in  <%s> computational sequence has wrong intervals array shape. " % (
						vid, root_name), error=False)
					failure = True
				# check the features next
				if (len(data[vid]["F"]) != 0 and len(data[vid]["F"].shape) != 2):
					if verbose: log.error(
						"Timestamped content (e.g. video,audio)  <%s> in  <%s> computational sequence has wrong features array shape. " % (
						vid, root_name), error=False)
					failure = True
			else:
				# check the intervals first - if failure simply show a warning - no exit since we want to identify all the cases
				if (len(data[vid]["intervals"]) != 0 and len(data[vid]["intervals"].shape) != 2):
					if verbose: log.error(
						"Timestamped content (e.g. video,audio) <%s> in  <%s> computational sequence has wrong intervals array shape. " % (
						vid, root_name), error=False)
					failure = True
				# check the features next
				if (len(data[vid]["features"]) != 0 and len(data[vid]["features"].shape) ==0):
					if verbose: log.error(
						"Timestamped content (e.g. video,audio)  <%s> in  <%s> computational sequence has wrong features array shape. " % (
						vid, root_name), error=False)
					failure = True
			pbar.update(1)
	#some other thing has happened! - RuntimeError
	except:
		if verbose:
			log.error("<%s> computational sequence data format could not be checked. "%root_name,error=True)
		pbar.close()
	pbar.close()

	#failure during intervals and features check
	if failure:
		log.error("<%s> computational sequence data integrity check failed due to inconsistency in intervals and features. "%root_name,error=True)
	else:
		log.success("<%s> computational sequence data in correct format." %root_name)
	return True


#this function checks the computatioanl sequence metadata. This will not crash the program if metadata is missing. If metadata is there the return value is simply True 
def validate_metadata_format(metadata,root_name,verbose=True):
	log.status("Checking the format of the metadata in <%s> computational sequence ..."%root_name)
	failure=False
	if type(metadata) is not dict:
		log.error("<%s> computational sequence metadata is not key-value pairs!", error=True)
	presenceFlag=[mtd in metadata.keys() for mtd in featuresetMetadataTemplate]
	#check if all the metadata is set
	if all (presenceFlag) is False:
		#verbose one is not set
		if verbose:
			missings=[x for (x,y) in zip (featuresetMetadataTemplate,presenceFlag) if y is False]
			log.error("Missing metadata in <%s> computational sequence: %s"%(root_name,str(missings)),error=False)
		failure=True
	if failure:
		log.error(msgstring="<%s> computational sequence does not have all the required metadata ... continuing "%root_name,error=False)
		return False
	else:
		log.success("<%s> computational sequence metadata in correct format."%root_name)
		return True





import re

DEFAULT_SENTENCE_BOUNDARIES = [r'(?<=[0-9]|[^0-9.])(\.)(?=[^0-9.]|[^0-9.]|[\s]|$)(?![\n\r]+)',
                               r'\.{2,}', r'\!+', r'\:+', r'\?+', r'[\n\r]+']
DEFAULT_PUNCTUATIONS = [r'(?<=[0-9]|[^0-9.])(\.)(?=[^0-9.]|[^0-9.]|[\s]|$)',
                        r'\.{2,}', r'\!+', r'\:+', r'\?+', r'\,+', r'\(|\)|\[|\]|\{|\}|\<|\>', r'\r\n|\r|\n']


class Document:
    """
    The Document is the main structure of our NLP process. It wraps the document being treated. A Document is composed of sentences and is iterable.
    Attributes
    ----------
    raw : str
        The raw text string passed as input to be sentencized.
    sentences : list of Sentence
        The list of sentences after sentencizing.

    """

    def __init__(self, document_text):
        """
        Parameters
        ----------
        document_text : str
            Text to be sentencized. Initialization immediately sentencizes the input text based on the input parameters. The sentences are also immediately tokenized.
        """

        self.raw = document_text
        self.sentences = sentencize(self.raw)
        self._index = 0

    def __getitem__(self, key):
        return self.sentences[key]

    def __repr__(self):
        return self.raw

    def __str__(self):
        return self.raw

    def __iter__(self):
        return self

    def __next__(self):
        if self._index < len(self.sentences):
            result = self.sentences[self._index]
            self._index += 1
            return result
        raise StopIteration

    def __len__(self):
        return len(self.sentences)


class Sentence:
    """
    Sentences are divisions of a Document. They are usually separated by punctuations. Sentences are divided into tokens. One can iterate over sentence tokens.
    Attributes
    ----------
    start_pos: int
        The starting position of the sentence in the raw Document.
    end_pos: int
        The ending position of the sentence in the raw Document. Includes punctuation position.
    previous_sentence: Sentence or None
        A pointer to the previous Sentence in a linked list manner. Prepared for future navigation.
    next_sentence: Sentence or None
        A pointer to the next Sentence in a linked list manner. Prepared for future navigation.
    tokens : list of Token
        The list of Tokens after tokenizing.

    Methods
    -------
    get: str
        Returns the string representation of the Sentence.

    """

    def __init__(self, start_position, end_position, raw_document_reference):
        """
        Parameters
        ----------
        start_position : int
            The starting position of the sentence in the raw Document.
        end_position : int
            The ending position of the sentence in the raw Document. Includes punctuation position.
        raw_document_reference: string
            The raw document string where the sentence is localized.
        """

        self.start_pos = int(start_position)
        self.end_pos = int(end_position)
        self._document_string = raw_document_reference
        self.next_sentence = None
        self.previous_sentence = None
        self.tokens = tokenize(self._document_string[self.start_pos:self.end_pos])
        self._index = 0

    def get(self):
        return self._document_string[self.start_pos:self.end_pos]

    def __getitem__(self, key):
        return self.tokens[key]

    def __repr__(self):
        return self.get()

    def __str__(self):
        return self.get()

    def __eq__(self, other):
        return self.get() == other

    def __iter__(self):
        return self

    def __next__(self):
        if self._index < len(self.tokens):
            result = self.tokens[self._index]
            self._index += 1
            return result
        raise StopIteration

    def __len__(self):
        return len(self.tokens)


class Token:
    """
    Tokens are divisions of a Sentence. They are usually separated by whitespaces. Punctuations are also tokens.
    Attributes
    ----------
    start_pos: int
        The starting position of the Token in the Sentence.
    end_pos: int
        The ending position of the Token in the Sentence.
    previous_token: Token or None
        A pointer to the previous Token in a linked list manner. Prepared for future navigation. First Token in a sentence is always SOS - acronym for Start of Sentence.
    next_token: Token or None
        A pointer to the next Token in a linked list manner. Prepared for future navigation. Last Token in a sentence is always EOS - acronym for End of Sentence.
    SOS: boolean
        Is the Token the start of a Sentence?
    EOS: boolean
        Is the Token the end of a Sentence?
    PoS: string or None
        The token predicted Part of Speech.
    Methods
    -------
    get: str
        Returns the string representation of the Token.
    """

    def __init__(self, start_position, end_position, raw_sentence_reference, SOS=False, EOS=False):
        """
        Parameters
        ----------
        start_position : int
            The starting position of the Token in the Sentence.
        end_position : int
            The ending position of the Token in the Sentence.
        raw_sentence_reference: str
            The Sentence string where the token is localized. It may be a substring of a raw Document representation.
        SOS: boolean, optional
            Creates a Start of Sentence Token. This influence Token representation and printing.
        EOS: boolean, optional
            Creates a End of Sentence Token. This influence Token representation and printing.
        """

        self.start_pos = int(start_position)
        self.end_pos = int(end_position)
        self._sentence_string = raw_sentence_reference
        self.next_token = None
        self.previous_token = None
        self.SOS = SOS
        self.EOS = EOS
        self.PoS = None
        self.raw = self._sentence_string[self.start_pos:self.end_pos]
        self.repr = self.raw

    def get(self):
        if self.SOS:
            return '<SOS>'
        elif self.EOS:
            return '<EOS>'
        else:
            return self.repr

    def __repr__(self):
        return self.get()

    def __str__(self):
        return self.get()

    def __eq__(self, other):
        return self.get() == other


## Static Functions

def sentencize(raw_input_document, sentence_boundaries=DEFAULT_SENTENCE_BOUNDARIES, delimiter_token='<SPLIT>'):
    """
    Sentencizes a string based on sentence boundaries. Returns a list of Sentences.

    Arguments:
        raw_input_document: str
            The raw input document in a string format.
        sentence_boundaries: list of str, optional
            A list of regex used to delimit sentence boundaries. Default regex includes correct period splitting, reticences, exclamation mark, question mark and colons.
            The default can be accessed by the global variable DEFAULT_SENTENCE_BOUNDARIES.
        delimiter_token: str, optional
            The token used for document segmentation. Usually an "agnostic" token. Defaults to <SPLIT>.
    Returns:
        list of Sentence: a list of sentences generated from raw_input_document.
    """
    if raw_input_document is None or raw_input_document == '':
        raise AttributeError("Empty document string passed as input. Please, verify your input.")
    working_document = raw_input_document
    punctuation_patterns = sentence_boundaries
    for punct in punctuation_patterns:
        working_document = re.sub(punct, r'\g<0>' + delimiter_token, working_document, flags=re.UNICODE)
    list_of_string_sentences = [x.strip(" ") for x in working_document.split(delimiter_token) if x.strip(" ") != ""]
    list_of_sentences = []
    previous = None
    for sent in list_of_string_sentences:
        start_pos = raw_input_document.find(sent)
        end_pos = start_pos + len(sent)
        new_sentence = Sentence(start_pos, end_pos, raw_input_document)
        list_of_sentences.append(new_sentence)
        if previous == None:
            previous = new_sentence
        else:
            previous.next_sentence = new_sentence
            new_sentence.previous_sentence = previous
            previous = new_sentence
    return list_of_sentences


def tokenize(raw_input_sentence, join_split_text=True, split_text_char=r'\-',
             punctuation_patterns=DEFAULT_PUNCTUATIONS, split_characters=r'[ \t]+', delimiter_token='<SPLIT>'):
    """
    Tokenizes a string based on token boundaries. Returns a list of Tokens.

    Arguments:
        raw_input_sentence: str
            The raw input Sentence in a string format.
        join_split_text: boolean, optional
            Whether to try to join multi-line text splits, like in the case of "sen-\ntence". Defaults to True.
        split_text_char: str, optional
            The split char used for checking and joining split strings. Defaults to hyphen.
        punctuation_patterns: list of str, optional
            A list of regex used to turn punctuations into tokens. Aside from sentence boundaries, also includes commas and parenthesis.
            The default can be accessed by the global variable DEFAULT_PUNCTUATIONS.
        split_characters: str, optional
            A string with regex for split characters. These are used to do tokenization after the sentence is preprocessed.
            Defaults to any whitespace (\s), any tab char (\t), newlines (\n) and carriage returns (\r).
        delimiter_token: str, optional
            The token used for sentence segmentation. Usually a 'agnostic' token. Defaults to <SPLIT>.

    Returns:
        list of Token: a list of tokens generated from raw_input_sentence, added by a starting SOS token and an ending EOS token.
    """
    if raw_input_sentence is None or raw_input_sentence == '':
        raise AttributeError("Empty sentence string passed as input. Please, verify your input.")
    working_sentence = raw_input_sentence
    # First deal with possible word splits:
    if join_split_text:
        working_sentence = re.sub(r'[a-z]+(' + split_text_char + r'[\n])[a-z]+', '', working_sentence)
    # Escape punctuation
    for punct in punctuation_patterns:
        working_sentence = re.sub(punct, r" \g<0> ", working_sentence)
    # Split at any split_characters
    working_sentence = re.sub(split_characters, delimiter_token, working_sentence)
    list_of_token_strings = [x.strip(" ") for x in working_sentence.split(delimiter_token) if x.strip(" ") != ""]
    previous = Token(0, 0, raw_input_sentence, SOS=True)
    list_of_tokens = [previous]
    for token in list_of_token_strings:
        start_pos = raw_input_sentence.find(token)
        end_pos = start_pos + len(token)
        new_token = Token(start_pos, end_pos, raw_input_sentence)
        list_of_tokens.append(new_token)
        previous.next_token = new_token
        new_token.previous_token = previous
        previous = new_token
    if previous.SOS != True:
        eos = Token(len(raw_input_sentence), len(raw_input_sentence), raw_input_sentence, EOS=True)
        previous.next_token = eos
        eos.previous_token = previous
        list_of_tokens.append(eos)
    return list_of_tokens


def untokenize(token_list):
    """
    Untokenizes a token list, turning it back to a string. Deals with SOS and EOS tokens, plus helps with correct punctuation addition.

    Arguments:
        token_list: list of Token
            The token list to be untokenized.

    Returns:
        string: the joint form of the Token list.

    Raises:
        TypeError: if the input is not a list;
        TypeError: if the input is not a list of Token.
    """
    if not isinstance(token_list, list):
        raise TypeError("Expected to recieve list of tokens. {} passed instead".format(type(token_list)))
    if len(token_list) < 1:
        return ""
    if not isinstance(token_list[0], Token):
        raise TypeError("Expected to recieve list of tokens. List of {} passed instead".format(type(token_list[0])))
    startpos = 0
    endpos = len(token_list)
    if len(token_list) < 3:
        return (" ".join([token.get() for token in token_list])).strip(" ")
    if token_list[0] == "<SOS>":
        startpos = 1
    if token_list[-1] == "<EOS>":
        endpos = -1
    punct = "!:?.;,\n"
    final_string = ""
    for token in token_list[startpos:endpos]:
        if not token.get() in punct:
            final_string += " "
        final_string += token.get()
    return final_string.strip(" ")



import mmkfeatures.models.image_text_fusion.char_embedding.train as tr_char
import mmkfeatures.models.image_text_fusion.conv_autoencoder.train as tr_conv
from mmkfeatures.models.image_text_fusion.char_embedding.char_cnn_rnn import CharCnnRnn
from mmkfeatures.models.image_text_fusion.conv_autoencoder.models import ConvAutoencoder
import torch
import cv2
import ntpath
import torchvision.transforms as transforms

class ImageTextFeaturesWrapper:
    def __init__(self):
        pass

    def train_image_conv_model(self,
                                   input_folder,output_folder,lr=0.001,epochs=100
                                   ):
        tr_conv.train(input_folder, output_folder, lr, epochs)

    def train_char_embedding_model(self,
                               json_path,output_folder,lr=0.001,epochs=100,model_type="fixed_gru",rnn_type="cvpr",img_tag="img_64x64_path"
                               ):

        tr_char.train(json_path=json_path, output_path=output_folder, learning_rate=float(lr), epochs=int(epochs),
              model_type=model_type, rnn_type=rnn_type, img_tag=img_tag)

    def prepare_text(self,string, max_str_len=201):
        # Converts a text description from string format to one-hot tensor format.
        labels = self.str_to_labelvec(string, max_str_len)
        one_hot = self.labelvec_to_onehot(labels)
        return one_hot

    def labelvec_to_onehot(self,labels):
        labels = torch.LongTensor(labels).unsqueeze(1)
        one_hot = torch.zeros(labels.size(0), 71, requires_grad=False).scatter_(1, labels, 1.)
        # Ignore zeros in one-hot mask (position 0 = empty one-hot)model.state_dict()
        one_hot = one_hot[:, 1:]
        one_hot = one_hot.permute(1, 0)
        return one_hot

    def str_to_labelvec(self,string, max_str_len=201):
        string = string.lower()
        alphabet = "abcdefghijklmnopqrstuvwxyz0123456789-,;.!?:'\"/\\|_@#$%^&*~`+-=<>()[]{} "
        # {'char': num, ...}
        alpha_to_num = {k: v + 1 for k, v in zip(alphabet, range(len(alphabet)))}
        labels = torch.zeros(max_str_len, requires_grad=False).long()
        max_i = min(max_str_len, len(string))
        for i in range(max_i):
            # Append ' ' number if char not found
            labels[i] = alpha_to_num.get(string[i], alpha_to_num[' '])
        return labels

    def get_embedding_text(self,embedding_path,text,output_path=None):
        device = torch.device('cuda' if torch.cuda.is_available() else 'cpu')
        embedding = CharCnnRnn()
        embedding.load_state_dict(torch.load(embedding_path))
        embedding = embedding.to(device)
        embeddings=[]
        torch.no_grad()
        embedding.eval()
        text = self.prepare_text(text)
        text = text.to(device)
        embedded_txt = embedding(text.unsqueeze(0))
        if output_path!=None :
            torch.save(embedded_txt, output_path)
        return embedded_txt

    def get_encode_image(img, encoder_path,image_path, output_file=None):
        device = torch.device('cuda' if torch.cuda.is_available() else 'cpu')
        encoder = ConvAutoencoder()
        encoder.load_state_dict(torch.load(encoder_path))
        encoder = encoder.to(device)
        torch.no_grad()
        encoder.eval()
        transform = transforms.ToTensor()
        image = cv2.imread(image_path, cv2.IMREAD_UNCHANGED)
        image = transform(image)
        image = image.float()
        image = image.to(device)
        enc_img = encoder(image.unsqueeze(0), encoder_mode=True)
        if output_file!=None:
            torch.save(enc_img, output_file)
        return enc_img











class MMFeaturesNode(dict):
    '''
    Multimodal Features Node Structure

    Content Id:
        name: a short name for the content
        format: '', a string indicating which formatio of the content
        text: '', a long text to describe the content
        modality: '', a string indicate which modality of the content
        feature_ids: [], ids for each features, if [] then using feature index in link representation
        feature_dim_names: [[]], if empty then using using index as dim name
        feature_extractor: a registered name to extract features from the original file
        features: an numpy array [[]]
        intervals: [[]] starting time and ending time, where [] indicates not specific
        space: [length, width,height], default value [] represents not specific
        labels: a list of labels, useful for multi-label or binary classification;a single label represent all feature has the same label
        origin: "" the original content file path, optional
        attributes: dict{}, user-defined key-value properties
        links: a list of relationships between the content's features, between inside and outside content
            e.g. link format: (feat_1,(rel_direction,rel_type,rel_func),feat_2),
                    where feat_1,feat_2 can be feature ids outside current content using format: content:feat1
    '''
    def __init__(self, content_id,meta_fields=None):
        super().__init__()
        self.data={}
        self["content_id"]=content_id
        if meta_fields!=None and type(meta_fields)==dict:
            for k in meta_fields.keys():
                self[k]=meta_fields[k]
            klist = list(self.keys())
            for k in klist:
                if type(self[k]) == str and (self[k] == "" or self[k] == None):
                    self.pop(k)
                elif type(self[k]) == list and len(self[k]) == 0:
                    self.pop(k)
                elif type(self[k]) == dict and len(self[k].keys()) == 0:
                    self.pop(k)
            # print(self)


    def get_content_id(self):
        return self["content_id"]

    def get_all_validated_keys(self):
        return ["content_id",'name','text','modality','feature_ids','feature_dim_names',
                'features','feature_extractor','intervals','space','labels','label_names',
                'origin','attributes','links',"format","raw","locations","objects"
                ]

    def get_all_validated_keys_alias(self):
        return ["ID", 'NA', 'TX', 'MD', 'FID', 'FDM',
                'F', 'FET', 'ITV', 'SP', 'LB', 'LBN',
                'OG', 'AT', 'LK', "FM","RAW","LOC","OBJ"
                ]

    def set_item(self,k,v):
        if k in self.get_all_validated_keys() or k in self.get_all_validated_keys_alias():
            self[k]=v
        else:
            raise Exception(f"{k} is not in validated key list!")

    def map_to_short_keys(self):
        temp_dict={}
        for k in self.keys():
            temp_dict[k]=self[k]
        for k in temp_dict.keys():
            index=self.get_all_validated_keys().index(k)
            self[self.get_all_validated_keys_alias()[index]]=temp_dict[k]
            self.pop(k)

    def validate_empty_field(self):
        klist=list(self.keys())
        for k in klist:
            if type(self[k]) == str and (self[k] == "" or self[k]==None) :
                self.pop(k)
            elif type(self[k]) == list and len(self[k]) == 0:
                self.pop(k)
            elif type(self[k]) == dict and len(self[k].keys()) == 0:
                self.pop(k)


    def get_item(self,k):
        return self[k]






import torch
import torch.nn as nn
import torch.nn.functional as F
from torch.autograd import Variable
import math
from functools import partial

__all__ = ['ResNeXt', 'resnet50', 'resnet101']


def conv3x3x3(in_planes, out_planes, stride=1):
    # 3x3x3 convolution with padding
    return nn.Conv3d(in_planes, out_planes, kernel_size=3,
                     stride=stride, padding=1, bias=False)


def downsample_basic_block(x, planes, stride):
    out = F.avg_pool3d(x, kernel_size=1, stride=stride)
    zero_pads = torch.Tensor(out.size(0), planes - out.size(1),
                             out.size(2), out.size(3),
                             out.size(4)).zero_()
    if isinstance(out.data, torch.cuda.FloatTensor):
        zero_pads = zero_pads.cuda()

    out = Variable(torch.cat([out.data, zero_pads], dim=1))

    return out


class ResNeXtBottleneck(nn.Module):
    expansion = 2

    def __init__(self, inplanes, planes, cardinality, stride=1, downsample=None):
        super(ResNeXtBottleneck, self).__init__()
        mid_planes = cardinality * int(planes / 32)
        self.conv1 = nn.Conv3d(inplanes, mid_planes, kernel_size=1, bias=False)
        self.bn1 = nn.BatchNorm3d(mid_planes)
        self.conv2 = nn.Conv3d(mid_planes, mid_planes, kernel_size=3, stride=stride,
                               padding=1, groups=cardinality, bias=False)
        self.bn2 = nn.BatchNorm3d(mid_planes)
        self.conv3 = nn.Conv3d(mid_planes, planes * self.expansion, kernel_size=1, bias=False)
        self.bn3 = nn.BatchNorm3d(planes * self.expansion)
        self.relu = nn.ReLU(inplace=True)
        self.downsample = downsample
        self.stride = stride

    def forward(self, x):
        residual = x

        out = self.conv1(x)
        out = self.bn1(out)
        out = self.relu(out)

        out = self.conv2(out)
        out = self.bn2(out)
        out = self.relu(out)

        out = self.conv3(out)
        out = self.bn3(out)

        if self.downsample is not None:
            residual = self.downsample(x)

        out += residual
        out = self.relu(out)

        return out


class ResNeXt(nn.Module):

    def __init__(self, block, layers, sample_size, sample_duration, shortcut_type='B', cardinality=32, num_classes=400, last_fc=True):
        self.last_fc = last_fc

        self.inplanes = 64
        super(ResNeXt, self).__init__()
        self.conv1 = nn.Conv3d(3, 64, kernel_size=7, stride=(1, 2, 2),
                               padding=(3, 3, 3), bias=False)
        self.bn1 = nn.BatchNorm3d(64)
        self.relu = nn.ReLU(inplace=True)
        self.maxpool = nn.MaxPool3d(kernel_size=(3, 3, 3), stride=2, padding=1)
        self.layer1 = self._make_layer(block, 128, layers[0], shortcut_type, cardinality)
        self.layer2 = self._make_layer(block, 256, layers[1], shortcut_type, cardinality, stride=2)
        self.layer3 = self._make_layer(block, 512, layers[2], shortcut_type, cardinality, stride=2)
        self.layer4 = self._make_layer(block, 1024, layers[3], shortcut_type, cardinality, stride=2)
        last_duration = math.ceil(sample_duration / 16)
        last_size = math.ceil(sample_size / 32)
        self.avgpool = nn.AvgPool3d((last_duration, last_size, last_size), stride=1)
        self.fc = nn.Linear(cardinality * 32 * block.expansion, num_classes)

        for m in self.modules():
            if isinstance(m, nn.Conv3d):
                n = m.kernel_size[0] * m.kernel_size[1] * m.out_channels
                m.weight.data.normal_(0, math.sqrt(2. / n))
            elif isinstance(m, nn.BatchNorm3d):
                m.weight.data.fill_(1)
                m.bias.data.zero_()

    def _make_layer(self, block, planes, blocks, shortcut_type, cardinality, stride=1):
        downsample = None
        if stride != 1 or self.inplanes != planes * block.expansion:
            if shortcut_type == 'A':
                downsample = partial(downsample_basic_block,
                                     planes=planes * block.expansion,
                                     stride=stride)
            else:
                downsample = nn.Sequential(
                    nn.Conv3d(self.inplanes, planes * block.expansion,
                              kernel_size=1, stride=stride, bias=False),
                    nn.BatchNorm3d(planes * block.expansion)
                )

        layers = []
        layers.append(block(self.inplanes, planes, cardinality, stride, downsample))
        self.inplanes = planes * block.expansion
        for i in range(1, blocks):
            layers.append(block(self.inplanes, planes, cardinality))

        return nn.Sequential(*layers)

    def forward(self, x):
        x = self.conv1(x)
        x = self.bn1(x)
        x = self.relu(x)
        x = self.maxpool(x)

        x = self.layer1(x)
        x = self.layer2(x)
        x = self.layer3(x)
        x = self.layer4(x)

        x = self.avgpool(x)

        x = x.view(x.size(0), -1)
        if self.last_fc:
            x = self.fc(x)

        return x

def get_fine_tuning_parameters(model, ft_begin_index):
    if ft_begin_index == 0:
        return model.parameters()

    ft_module_names = []
    for i in range(ft_begin_index, 5):
        ft_module_names.append('layer{}'.format(ft_begin_index))
    ft_module_names.append('fc')

    parameters = []
    for k, v in model.named_parameters():
        for ft_module in ft_module_names:
            if ft_module in k:
                parameters.append({'params': v})
                break
        else:
            parameters.append({'params': v, 'lr': 0.0})

    return parameters

def resnet50(**kwargs):
    """Constructs a ResNet-50 model.
    """
    model = ResNeXt(ResNeXtBottleneck, [3, 4, 6, 3], **kwargs)
    return model

def resnet101(**kwargs):
    """Constructs a ResNet-101 model.
    """
    model = ResNeXt(ResNeXtBottleneck, [3, 4, 23, 3], **kwargs)
    return model

def resnet152(**kwargs):
    """Constructs a ResNet-101 model.
    """
    model = ResNeXt(ResNeXtBottleneck, [3, 8, 36, 3], **kwargs)
    return model



import glob
import cv2

import torch.nn as nn
import torch.nn.functional as F
from torch.utils.data import Dataset


# define the NN architecture
class ConvAutoencoder(nn.Module):
    def __init__(self):
        super(ConvAutoencoder, self).__init__()
        # encoder layers ##
        self.conv1 = nn.Conv2d(1, 16, 3, padding=1)
        self.conv2 = nn.Conv2d(16, 4, 3, padding=1)
        self.pool = nn.MaxPool2d(2, 2)
        self.adapt_pool = nn.AdaptiveMaxPool2d((16, 16))

        # decoder layers ##
        self.t_conv1 = nn.ConvTranspose2d(4, 16, 2, stride=2)
        self.t_conv2 = nn.ConvTranspose2d(16, 1, 2, stride=2)

    def forward(self, x, encoder_mode=False):
        # encode ##
        x = F.relu(self.conv1(x))
        x = self.pool(x)
        x = F.relu(self.conv2(x))
        x = self.pool(x)

        x_encoder = self.adapt_pool(x)

        if encoder_mode:
            return x_encoder
        else:
            # decode ##
            x = F.relu(self.t_conv1(x))
            x = F.sigmoid(self.t_conv2(x))
            return x, x_encoder


class ImgDataset(Dataset):
    def __init__(self, data_root, transform, device, img_format='.jpg'):
        super(ImgDataset, self).__init__()
        self.sample = []
        self.transform = transform
        self.device = device
        self.img_format = img_format
        print(f"ImgDataset load images from - {data_root + '/*' + img_format}")
        for img in glob.glob(data_root + '/*' + img_format):
            image = cv2.imread(img, cv2.IMREAD_UNCHANGED)
            if image is None:
                continue
            image = self.transform(image)
            image = image.float()
            image = image.to(self.device)
            self.sample.append(image)

    def __len__(self):
        return len(self.sample)

    def __getitem__(self, index):
        return self.sample[index]



#CMU Multimodal SDK, CMU Multimodal Model SDK

#Multimodal Language Analysis in the Wild: CMU-MOSEI Dataset and Interpretable Dynamic Fusion Graph, Amir Zadeh, Paul Pu Liang, Jonathan Vanbriesen, Soujanya Poria, Edmund Tong, Erik Cambria, Minghai Chen, Louis-Philippe Morency - http://www.aclweb.org/anthology/P18-1208

#pattern_model: a nn.Sequential model which will be used as core of the models inside the DFG

#in_dimensions: input dimensions of each modality

#out_dimension: output dimension of the pattern models

#efficacy_model: the core of the efficacy model

#in_modalities: inputs from each modality, the same order as in_dimensions

import torch
import time
from torch import nn
import torch.nn.functional as F
import copy
from six.moves import reduce
from itertools import chain,combinations
from collections import OrderedDict

class DynamicFusionGraph(nn.Module):

	def __init__(self,pattern_model,in_dimensions,out_dimension,efficacy_model):
		super(DynamicFusionGraph, self).__init__()

		self.num_modalities=len(in_dimensions)
		self.in_dimensions=in_dimensions
		self.out_dimension=out_dimension

		#in this part we sort out number of connections, how they will be connected etc.
		self.powerset=list(chain.from_iterable(combinations(range(self.num_modalities), r) for r in range(self.num_modalities+1)))[1:]

		#initializing the models inside the DFG
		self.input_shapes={tuple([key]):value for key,value in zip(range(self.num_modalities),in_dimensions)}
		self.networks={}
		self.total_input_efficacies=0
		for key in self.powerset[self.num_modalities:]:
			#connections coming from the unimodal components
			unimodal_dims=0
			for modality in key:
				unimodal_dims+=in_dimensions[modality]
			multimodal_dims=((2**len(key)-2)-len(key))*out_dimension
			self.total_input_efficacies+=2**len(key)-2
			#for the network that outputs key component, what is the input dimension
			final_dims=unimodal_dims+multimodal_dims
			self.input_shapes[key]=final_dims
			pattern_copy=copy.deepcopy(pattern_model)
			final_model=nn.Sequential(*[nn.Linear(self.input_shapes[key],list(pattern_copy.children())[0].in_features),pattern_copy])
			self.networks[key]=final_model
		#finished construction weights, now onto the t_network which summarizes the graph
		self.total_input_efficacies+=2**self.num_modalities-1
		self.t_in_dimension=unimodal_dims+(2**self.num_modalities-(self.num_modalities)-1)*out_dimension
		pattern_copy=copy.deepcopy(pattern_model)
		self.t_network=nn.Sequential(*[nn.Linear(self.t_in_dimension,list(pattern_copy.children())[0].in_features),pattern_copy])
		self.efficacy_model=nn.Sequential(*[nn.Linear(sum(in_dimensions),list(efficacy_model.children())[0].in_features),efficacy_model,nn.Linear(list(efficacy_model.children())[-1].out_features,self.total_input_efficacies)])
	
	def __call__(self,in_modalities):
		return self.fusion(in_modalities)	

	def fusion(self,in_modalities):

		bs=in_modalities[0].shape[0]
		outputs={}
		for modality,index in zip(in_modalities,range(len(in_modalities))):
			outputs[tuple([index])]=modality
		
		efficacies=self.efficacy_model(torch.cat([x for x in in_modalities],dim=1))
		efficacy_index=0
		for key in self.powerset[self.num_modalities:]:
			small_power_set=list(chain.from_iterable(combinations(key, r) for r in range(len(key)+1)))[1:-1]
			this_input=torch.cat([outputs[x]*efficacies[:,efficacy_index+y].view(-1,1) for x,y in zip(small_power_set,range(len(small_power_set)))],dim=1)
			outputs[key]=self.networks[key](this_input)
			efficacy_index+=len(small_power_set)

		small_power_set.append(tuple(range(self.num_modalities)))
		t_input=torch.cat([outputs[x]*efficacies[:,efficacy_index+y].view(-1,1) for x,y in zip(small_power_set,range(len(small_power_set)))],dim=1)
		t_output=self.t_network(t_input)
		return t_output,outputs,efficacies

	def forward(self, x):
		print("Not yet implemented for nn.Sequential")
		exit(-1)

if __name__=="__main__":
	print("This is a module and hence cannot be called directly ...")
	print("A toy sample will now run ...")
	
	from torch.autograd import Variable
	import torch.nn.functional as F
	import numpy

	inputx=Variable(torch.Tensor(numpy.array(numpy.zeros([32,40]))),requires_grad=True)
	inputy=Variable(torch.Tensor(numpy.array(numpy.zeros([32,12]))),requires_grad=True)
	inputz=Variable(torch.Tensor(numpy.array(numpy.zeros([32,20]))),requires_grad=True)
	inputw=Variable(torch.Tensor(numpy.array(numpy.zeros([32,25]))),requires_grad=True)
	modalities=[inputx,inputy,inputz,inputw]
	
	#a simple linear function without any activations
	pattern_model=nn.Sequential(nn.Linear(100,20))
	efficacy_model=nn.Sequential(nn.Linear(100,20))
	fmodel=DynamicFusionGraph(pattern_model,[40,12,20,25],20,efficacy_model)
	
	out=fmodel(modalities)
	print("Output")
	print(out[0].shape,out[2].shape)
	print("Toy sample finished ...")




import pickle
import unicodedata
from functools import reduce

# List Of English Stop Words
# http://armandbrahaj.blog.al/2009/04/14/list-of-english-stop-words/
_WORD_MIN_LENGTH = 3
_STOP_WORDS = frozenset([
'a', 'about', 'above', 'above', 'across', 'after', 'afterwards', 'again',
'against', 'all', 'almost', 'alone', 'along', 'already', 'also','although',
'always','am','among', 'amongst', 'amoungst', 'amount',  'an', 'and', 'another',
'any','anyhow','anyone','anything','anyway', 'anywhere', 'are', 'around', 'as',
'at', 'back','be','became', 'because','become','becomes', 'becoming', 'been',
'before', 'beforehand', 'behind', 'being', 'below', 'beside', 'besides',
'between', 'beyond', 'bill', 'both', 'bottom','but', 'by', 'call', 'can',
'cannot', 'cant', 'co', 'con', 'could', 'couldnt', 'cry', 'de', 'describe',
'detail', 'do', 'done', 'down', 'due', 'during', 'each', 'eg', 'eight',
'either', 'eleven','else', 'elsewhere', 'empty', 'enough', 'etc', 'even',
'ever', 'every', 'everyone', 'everything', 'everywhere', 'except', 'few',
'fifteen', 'fify', 'fill', 'find', 'fire', 'first', 'five', 'for', 'former',
'formerly', 'forty', 'found', 'four', 'from', 'front', 'full', 'further', 'get',
'give', 'go', 'had', 'has', 'hasnt', 'have', 'he', 'hence', 'her', 'here',
'hereafter', 'hereby', 'herein', 'hereupon', 'hers', 'herself', 'him',
'himself', 'his', 'how', 'however', 'hundred', 'ie', 'if', 'in', 'inc',
'indeed', 'interest', 'into', 'is', 'it', 'its', 'itself', 'keep', 'last',
'latter', 'latterly', 'least', 'less', 'ltd', 'made', 'many', 'may', 'me',
'meanwhile', 'might', 'mill', 'mine', 'more', 'moreover', 'most', 'mostly',
'move', 'much', 'must', 'my', 'myself', 'name', 'namely', 'neither', 'never',
'nevertheless', 'next', 'nine', 'no', 'nobody', 'none', 'noone', 'nor', 'not',
'nothing', 'now', 'nowhere', 'of', 'off', 'often', 'on', 'once', 'one', 'only',
'onto', 'or', 'other', 'others', 'otherwise', 'our', 'ours', 'ourselves', 'out',
'over', 'own','part', 'per', 'perhaps', 'please', 'put', 'rather', 're', 'same',
'see', 'seem', 'seemed', 'seeming', 'seems', 'serious', 'several', 'she',
'should', 'show', 'side', 'since', 'sincere', 'six', 'sixty', 'so', 'some',
'somehow', 'someone', 'something', 'sometime', 'sometimes', 'somewhere',
'still', 'such', 'system', 'take', 'ten', 'than', 'that', 'the', 'their',
'them', 'themselves', 'then', 'thence', 'there', 'thereafter', 'thereby',
'therefore', 'therein', 'thereupon', 'these', 'they', 'thickv', 'thin', 'third',
'this', 'those', 'though', 'three', 'through', 'throughout', 'thru', 'thus',
'to', 'together', 'too', 'top', 'toward', 'towards', 'twelve', 'twenty', 'two',
'un', 'under', 'until', 'up', 'upon', 'us', 'very', 'via', 'was', 'we', 'well',
'were', 'what', 'whatever', 'when', 'whence', 'whenever', 'where', 'whereafter',
'whereas', 'whereby', 'wherein', 'whereupon', 'wherever', 'whether', 'which',
'while', 'whither', 'who', 'whoever', 'whole', 'whom', 'whose', 'why', 'will',
'with', 'within', 'without', 'would', 'yet', 'you', 'your', 'yours', 'yourself',
'yourselves', 'the'])

class InvertedIndex:
    def __init__(self):
        pass

    def word_split(self,text):
        """
        Split a text in words. Returns a list of tuple that contains
        (word, location) location is the starting byte position of the word.
        """
        word_list = []
        wcurrent = []
        windex = None
        if type(text)!=str:
            text=text.decode(encoding='UTF-8',errors='ignore')

        for i, c in enumerate(text):
            if str(c).isalnum():
                wcurrent.append(str(c))
                windex = i
            elif wcurrent:
                word = u''.join(wcurrent)
                word_list.append((windex - len(word) + 1, word))
                wcurrent = []

        if wcurrent:
            word = u''.join(wcurrent)
            word_list.append((windex - len(word) + 1, word))
        # print(word_list)
        return word_list

    def words_cleanup(self,words):
        """
        Remove words with length less then a minimum and stopwords.
        """
        cleaned_words = []
        for index, word in words:
            if len(word) < _WORD_MIN_LENGTH or word in _STOP_WORDS:
                continue
            cleaned_words.append((index, word))
        return cleaned_words

    def words_normalize(self,words):
        """
        Do a normalization precess on words. In this case is just a tolower(),
        but you can add accents stripping, convert to singular and so on...
        """
        normalized_words = []
        for index, word in words:
            wnormalized = word.lower()
            normalized_words.append((index, wnormalized))
        return normalized_words

    def word_index(self,text):
        """
        Just a helper method to process a text.
        It calls word split, normalize and cleanup.
        """
        words = self.word_split(text)
        words = self.words_normalize(words)
        words = self.words_cleanup(words)
        return words

    def inverted_index(self,text):
        """
        Create an Inverted-Index of the specified text document.
            {word:[locations]}
        """
        inverted = {}

        for index, word in self.word_index(text):
            # print(index,word)
            locations = inverted.setdefault(word, [])
            locations.append(index)

        return inverted

    def inverted_index_add(self,inverted, doc_id, doc_index):
        """
        Add Invertd-Index doc_index of the document doc_id to the
        Multi-Document Inverted-Index (inverted),
        using doc_id as document identifier.
            {word:{doc_id:[locations]}}
        """
        for word, locations in doc_index.items():
            indices = inverted.setdefault(word, {})
            indices[doc_id] = locations
        return inverted

    def search(self,index_file, query):
        """
        Returns a set of documents id that contains all the words in your query.
        """
        inverted=pickle.load(open(index_file,"rb"))
        words = [word for _, word in self.word_index(query) if word in inverted]
        results = [set(inverted[word].keys()) for word in words]
        return reduce(lambda x, y: x & y, results) if results else []

    def create(self,index_file,inverted_index_file):
        list_id_text = pickle.load(open(index_file, 'rb'))
        # Build Inverted-Index for documents
        inverted = {}
        documents = {}
        for item in list_id_text:
            documents[item[0]] = item[1]

        for doc_id, text in documents.items():
            doc_index = self.inverted_index(text)
            self.inverted_index_add(inverted, doc_id, doc_index)

        pickle.dump(inverted,open(inverted_index_file,"wb"))

    def create_with_data(self,index_data,inverted_index_file):
        # Build Inverted-Index for documents
        inverted = {}
        documents = {}
        for item in index_data:
            documents[item[0]] = item[1]

        for doc_id, text in documents.items():
            # print("docId="+str(doc_id),text)
            doc_index = self.inverted_index(text)
            # print(doc_index)
            self.inverted_index_add(inverted, doc_id, doc_index)
            # print()

        pickle.dump(inverted,open(inverted_index_file,"wb"))

    def test(self,inverted_index_file):
        import time
        # Print Inverted-Index
        # for word, doc_locations in inverted.items():
        #    print(word, doc_locations)
        inverted=pickle.load(open(inverted_index_file,"rb"))

        # Search something and print results
        start_time = time.time()
        queries = ['gram-negative bacteria', 'Fungal infection', 'purulent exudate']
        for query in queries:
            result_docs = self.search(inverted, query)
            print("Search for '%s': %r" % (query, result_docs))
            '''
            for _, word in word_index(query):
                def extract_text(doc, index):
                    return documents[doc][index:index+40].replace('\n', ' ')

                for doc in result_docs:
                    for index in inverted[word][doc]:
                        print('   - '+doc+' %s...' % extract_text(doc, index))
            print()
            '''
            print(result_docs)
        end_time = time.time()
        time_passed = round((end_time - start_time) * 1000, 2)
        print("time:", time_passed, "ms")
        print()











import torch as th
import math
import numpy as np
from video_loader import VideoLoader
from torch.utils.data import DataLoader
import argparse
from model import get_model_p
from preprocessing import Preprocessing
from random_sequence_shuffler import RandomSequenceSampler
import torch.nn.functional as F
import ffmpeg

# parameter setting
framerate=1
size=112
centercrop=False
type="2d" # 3d
batch_size=4
l2_normalize=1
half_precision=1
resnext101_model_path="model/resnext101.pth"
num_decoding_thread=4

def _get_video_dim( video_path):
    probe = ffmpeg.probe(video_path)
    video_stream = next((stream for stream in probe['streams']
                         if stream['codec_type'] == 'video'), None)
    width = int(video_stream['width'])
    height = int(video_stream['height'])
    return height, width

def _get_output_dim( h, w):
    if isinstance(size, tuple) and len(size) == 2:
        return size
    elif h >= w:
        return int(h * size / w), size
    else:
        return size, int(w * size / h)

def load_video(video_path):
    try:
        # video_path = ""
        output_file = ""
        h, w = _get_video_dim(video_path)
    except:
        print('ffprobe failed at: {}'.format(video_path))

    height, width = _get_output_dim(h, w)
    cmd = (
        ffmpeg
            .input(video_path)
            .filter('fps', fps=framerate)
            .filter('scale', width, height)
    )
    if centercrop:
        x = int((width - size) / 2.0)
        y = int((height - size) / 2.0)
        cmd = cmd.crop(x, y, size, size)
    out, _ = (
        cmd.output('pipe:', format='rawvideo', pix_fmt='rgb24')
            .run(capture_stdout=True, quiet=True)
    )
    if centercrop and isinstance(size, int):
        height, width = size, size
    video = np.frombuffer(out, np.uint8).reshape([-1, height, width, 3])
    video = th.from_numpy(video.astype('float32'))
    video = video.permute(0, 3, 1, 2)
    return video

if __name__=="__main__":
    video_file = "../dataset/videos/a.mp4"
    output_file = "../dataset/output/single_features.npy"

    preprocess = Preprocessing(type)
    model = get_model_p(type,resnext101_model_path)

    with th.no_grad():
        data = load_video(video_file)

        video = data.squeeze()
        print("len(video.shape)=", len(video.shape))

        video = preprocess(video)

        n_chunk = len(video)
        print("n_chunk = ", n_chunk)
        print("batch size = ", batch_size)
        features = th.cuda.FloatTensor(n_chunk, 2048).fill_(0)
        n_iter = int(math.ceil(n_chunk / float(batch_size)))
        print("number of inter. = ", n_iter)
        for i in range(n_iter):
            min_ind = i * batch_size
            max_ind = (i + 1) * batch_size
            video_batch = video[min_ind:max_ind].cuda()
            batch_features = model(video_batch)
            if l2_normalize:
                batch_features = F.normalize(batch_features, dim=1)
            features[min_ind:max_ind] = batch_features
        features = features.cpu().numpy()
        print("features.shape = ", features.shape)
        if half_precision:
            features = features.astype('float16')
        print("save...", output_file)
        print(features.shape)
        np.save(output_file, features)
        print()




import h5py
import time
import requests
from tqdm import tqdm 
import os
import math
import sys
from mmkfeatures.fusion  import log


def read_URL(url,destination):
	if destination is None:
		log.error("Destination is not specified when downloading data",error=True)

	#if os.path.isdir(destination.rsplit(os.sep,1)[-2]) is False:
	#	os.mkdir(destination.rsplit(os.sep,1)[-2])

	if(os.path.isfile(destination)):
		log.error("%s file already exists ..."%destination,error=True)

	r = requests.get(url, stream=True)
	if r.status_code != 200:
		log.error('URL: %s does not exist'%url,error=True) 
	# Total size in bytes.
	total_size = int(r.headers.get('content-length', 0)); 
	block_size = 1024
	unit=total_size/block_size
	wrote = 0
	if ":" in destination:
		ds = destination.split("http://")
		destination = ds[0] + "http___" + ds[1].replace("/", "_")
	with open(destination, 'wb') as f:
		log.status("Downloading from %s to %s..."%(url,destination))
		pbar=log.progress_bar(total=math.ceil(total_size//block_size),data=r.iter_content(block_size),postfix="Total in kBs",unit='kB', leave=False)
		for data in pbar:#unit_scale=True,
			wrote = wrote  + len(data)
			f.write(data)
	pbar.close()

	if total_size != 0 and wrote != total_size:
		log.error("Error downloading the data to %s ..."%destination,error=True)

	log.success("Download complete!")
	return True

if __name__=="__main__":
	readURL("http://immortal.multicomp.cs.cmu.edu/CMU-MOSEI/acoustic/CMU_MOSEI_COVAREP.csd","./hi.csd")
